import os
from tkinter import *
from tkinter import filedialog
import csv


def read_toschedule():

    to_schedules = []
    actual_request_dict = { 0: False, 1: True}

    # this is a window prompt for selecting csv file to be read
    dialog_root = Tk()
    dialog_root.geometry("1200x700+350+150")
    dialog_root.withdraw()

    # this will obtain file path of the selected csv file
    file_path = filedialog.askopenfilename(title='Please select the to be scheduled .csv file')
    file_path = os.path.normpath(file_path)

    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        next(mock_csv, None)  # skip header
        for row in mock_csv: # add each tuple into the list
            to_schedules.append((float(row[1]), row[2], actual_request_dict[int(row[3])]))

    return to_schedules


import numpy as np
import numpy as np
from Generate_initial_solution import initial_solution_first_fit
from Generate_initial_solution import initial_solution_random_fit
from MO_fitting_func_calc import MO_calculate
from MO_fitting_func_calc import calculate_delta_S
from MO_fitting_func_calc import calc_crowding_distance
import find_legal_neighbors as fd
import swapper as sp
import pareto as pt
from read_MAS import read_mas
from read_csv import read_csv
from read_toschedule import read_toschedule
from plot_pareto_front import plot_front
import time
import random
from view_timetable import show_timetable
from threading import Thread


def schedule_SA(ini_temp=1000, cool_rate=0.99, termi_num=30):
	day_of_week_indexes = {0: 'Monday1',
	                       1: 'Tuesday1',
	                       2: 'Wednesday1',
	                       3: 'Thursday1',
	                       4: 'Friday1',
	                       5: 'Monday2',
	                       6: 'Tuesday2',
	                       7: 'Wednesday2',
	                       8: 'Thursday2',
	                       9: 'Friday2',
	                       10: 'Monday3',
	                       11: 'Tuesday3',
	                       12: 'Wednesday3',
	                       13: 'Thursday3',
	                       14: 'Friday3',
	                       }
	OT_indexes = {0: 'L1',
	              1: 'L2',
	              2: 'L3',
	              3: 'L4',
	              4: 'L5',
	              5: 'L6',
	              6: 'L7',
	              7: 'L8',
	              8: 'M1',
	              9: 'M2',
	              10: 'M3',
	              11: 'M4',
	              12: 'M5',
	              13: 'OT 24',
	              14: 'OT 25',
	              15: 'OT 22',
	              16: 'R1',
	              17: 'R4',
	              18: 'R5',
	              19: 'R6',
	              20: 'R7',
	              21: 'R8',
	              22: 'MRI',
	              }
	current_schedule = read_csv()
	schedule_days = current_schedule[1]
	day_index = current_schedule[3]
	current_schedule = current_schedule[0]
	to_schedule = read_toschedule()
	MAS_full = read_mas()
	MAS_allowed = MAS_full['OTO']

	t0 = time.time()
	# first fit or random fit
	soln = initial_solution_random_fit(current_schedule, to_schedule, MAS_allowed, day_index, OT_indexes)
	# SIMULATED ANNEALING ALGORITHM
	print("SIMULATED ANNEALING Algorithm")
	T = ini_temp  # higher temperature, higher chance of accepting bad solution (more diversed solution)
	alpha = cool_rate  # cooling rate
	termination_number = termi_num  # how many times of reaching the same fitting value to stop the run?
	termination_count = 0

	while termination_count < termination_number:
		sp.pushback2(soln)
		curr_score = MO_calculate(soln)
		legal_swaps = fd.all_legal_swaps(soln, to_schedule, MAS_allowed, day_index, OT_indexes)
		neighbour = random.choice(legal_swaps)
		perturb_soln = sp.swapping(soln, neighbour)
		perturb_score = MO_calculate(perturb_soln)
		before = curr_score
		if np.all(perturb_score <= curr_score):
			soln = perturb_soln
			curr_score = perturb_score
		else:  # if does not dominate, select based on some probability, UMOSA
			p = calculate_delta_S(soln, perturb_soln, T)
			if p > random.uniform(0, 1):
				soln = perturb_soln
				curr_score = perturb_score

		# check improvement for termination
		improvement = before - curr_score
		if np.all(improvement==0):
			termination_count += 1
		else:
			termination_count = 0

		print("The current score is:" + str(curr_score))
		T = T * alpha
	t1 = time.time()
	print("Time Taken for SA: " + str(t1-t0) + ' seconds')

	sol_list = [soln, schedule_days]
	thread = Thread(target=show_timetable, args=[sol_list])
	thread.start()
	print(curr_score)


import os
import csv
from tkinter import *
from tkinter import filedialog
from datetime import *

def read_toschedule():

    # this is a window prompt for selecting csv file to be read
    dialog_root = Tk()
    dialog_root.geometry("1200x700+350+150")
    dialog_root.withdraw()

    # this will obtain file path of the selected csv file
    file_path = filedialog.askopenfilename(title='Please select the to be scheduled .csv file')
    file_path = os.path.normpath(file_path)

    # discipline dictionary to identify code
    discipline_codes = {10: 'BREAST',
                        11: 'OTO',
                        12: 'NES',
                        13: 'CLR',
                        14: 'HPB',
                        15: 'PLS',
                        16: 'ENT',
                        17: 'SUR-ONCO',
                        18: 'UGI',
                        19: 'H&N',
                        20: 'O&G',
                        21: 'HND',
                        22: 'OMS',
                        23: 'VAS',
                        24: 'CTS',
                        25: 'URO'
                        }

    proc_codes_oto = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    key_list = list(discipline_codes.keys())
    value_list = list(discipline_codes.values())
    proc_key_list = list(proc_codes_oto.keys())
    proc_value_list = list(proc_codes_oto.values())

    procedure_list = []
    to_schedule = []

    # find number of surgeries in the to_schedule csv file
    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        next(mock_csv, None)  # skip header
        for row in mock_csv: # add each array
            duration = int(float(row[1]) / 0.25)
            duplicate_count = procedure_list.count(row[2])
            procedure_list.append(row[2])
            position = value_list.index(row[0])
            proc_position = proc_value_list.index(row[2])
            if int(row[3]) == 1:
                temp_list = [1100000 + (key_list[position])*1000 + (proc_key_list[proc_position])*10 + duplicate_count] * duration
                to_schedule.append(temp_list)
            elif int(row[3]) == 0:
                temp_list = [1200000 + (key_list[position])*1000 + (proc_key_list[proc_position])*10 + duplicate_count] * duration
                to_schedule.append(temp_list)

    return to_schedule


def read_toschedule1(file_path):

    # this is a window prompt for selecting csv file to be read
    dialog_root = Tk()
    dialog_root.geometry("1200x700+350+150")
    dialog_root.withdraw()

    # discipline dictionary to identify code
    discipline_codes = {10: 'BREAST',
                        11: 'OTO',
                        12: 'NES',
                        13: 'CLR',
                        14: 'HPB',
                        15: 'PLS',
                        16: 'ENT',
                        17: 'SUR-ONCO',
                        18: 'UGI',
                        19: 'H&N',
                        20: 'O&G',
                        21: 'HND',
                        22: 'OMS',
                        23: 'VAS',
                        24: 'CTS',
                        25: 'URO'
                        }

    proc_codes_oto = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    key_list = list(discipline_codes.keys())
    value_list = list(discipline_codes.values())
    proc_key_list = list(proc_codes_oto.keys())
    proc_value_list = list(proc_codes_oto.values())

    procedure_list = []
    to_schedule = []

    # find number of surgeries in the to_schedule csv file
    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        next(mock_csv, None)  # skip header
        for row in mock_csv:  # add each array
            duration = int(float(row[1]) / 0.25)
            duplicate_count = procedure_list.count(row[2])
            procedure_list.append(row[2])
            position = value_list.index(row[0])
            proc_position = proc_value_list.index(row[2])
            if int(row[3]) == 1:
                temp_list = [1100000 + (key_list[position]) * 1000 + (
                proc_key_list[proc_position]) * 10 + duplicate_count] * duration
                to_schedule.append(temp_list)
            elif int(row[3]) == 0:
                temp_list = [1200000 + (key_list[position]) * 1000 + (
                proc_key_list[proc_position]) * 10 + duplicate_count] * duration
                to_schedule.append(temp_list)

    return to_schedule


from tkinter import *
from datetime import *

root = Tk()

# set title and size of the GUI window
root.title('Timetable')
root.geometry("1200x700+350+150")

# set secondary frame position and size (center right)
frame = Frame(root, bg='blue')
frame.place(relx=0.05, rely=0.05, relwidth=0.9, relheight=0.9)

# get today's date and set as heading
today = date.today()
if today.weekday() == 5:
    today = date.today() - timedelta(days=1)
if today.weekday() == 6:
    today = date.today() - timedelta(days=2)
week_day = today.strftime("%A")

# place main date label
heading = Label(frame, text= week_day + " , " + str(today), bg='#E5E7E9', relief=RIDGE)
heading.place(relx=0.38, relwidth=0.25, y=0)

# set secondary frame position and size
canvas = Canvas(root, bg='red')
canvas.place(relx=0.1, rely=0.1, relwidth=0.8, relheight=0.8)

# create and set vertical scrollbar
vbar = Scrollbar(frame)
vbar.config(command=canvas.yview)
canvas.config(yscrollcommand=vbar.set)
vbar.pack(side="right", fill=Y)

# create a frame inside the canvas
# configure size of frame
canvas_frame = Frame(canvas, width=955, height=900, bg='green')
canvas_frame.pack(expand=True, fill=BOTH)

canvas_id = canvas.create_window(0, 0, window=canvas_frame, anchor="nw")

# time-table headings
label1 = Label(canvas_frame, text='Time:', bg='#D0ECE7', relief=RIDGE)
label2 = Label(canvas_frame, text='L1', bg='#D0ECE7', relief=RIDGE)
label3 = Label(canvas_frame, text='L2', bg='#D0ECE7', relief=RIDGE)
label4 = Label(canvas_frame, text='L3', bg='#D0ECE7', relief=RIDGE)
label5 = Label(canvas_frame, text='L4', bg='#D0ECE7', relief=RIDGE)
label6 = Label(canvas_frame, text='L5', bg='#D0ECE7', relief=RIDGE)
label7 = Label(canvas_frame, text='L6', bg='#D0ECE7', relief=RIDGE)
label8 = Label(canvas_frame, text='L7', bg='#D0ECE7', relief=RIDGE)
label9 = Label(canvas_frame, text='L8', bg='#D0ECE7', relief=RIDGE)
label10 = Label(canvas_frame, text='M1', bg='#D0ECE7', relief=RIDGE)
label11 = Label(canvas_frame, text='M2', bg='#D0ECE7', relief=RIDGE)
label12 = Label(canvas_frame, text='M3', bg='#D0ECE7', relief=RIDGE)
label13 = Label(canvas_frame, text='M4', bg='#D0ECE7', relief=RIDGE)
label14 = Label(canvas_frame, text='M5', bg='#D0ECE7', relief=RIDGE)
label15 = Label(canvas_frame, text='OT 24', bg='#D0ECE7', relief=RIDGE)
label16 = Label(canvas_frame, text='OT 25', bg='#D0ECE7', relief=RIDGE)
label17 = Label(canvas_frame, text='OT 22', bg='#D0ECE7', relief=RIDGE)
label18 = Label(canvas_frame, text='R1', bg='#D0ECE7', relief=RIDGE)
label19 = Label(canvas_frame, text='R4', bg='#D0ECE7', relief=RIDGE)
label20 = Label(canvas_frame, text='R5', bg='#D0ECE7', relief=RIDGE)
label21 = Label(canvas_frame, text='R6', bg='#D0ECE7', relief=RIDGE)
label22 = Label(canvas_frame, text='R7', bg='#D0ECE7', relief=RIDGE)
label23 = Label(canvas_frame, text='R8', bg='#D0ECE7', relief=RIDGE)
label24 = Label(canvas_frame, text='MRI', bg='#D0ECE7', relief=RIDGE)
label1.place(relwidth=0.08)
label2.place(relwidth=0.04, relx=0.08)
label3.place(relwidth=0.04, relx=0.12)
label4.place(relwidth=0.04, relx=0.16)
label5.place(relwidth=0.04, relx=0.20)
label6.place(relwidth=0.04, relx=0.24)
label7.place(relwidth=0.04, relx=0.28)
label8.place(relwidth=0.04, relx=0.32)
label9.place(relwidth=0.04, relx=0.36)
label10.place(relwidth=0.04, relx=0.40)
label11.place(relwidth=0.04, relx=0.44)
label12.place(relwidth=0.04, relx=0.48)
label13.place(relwidth=0.04, relx=0.52)
label14.place(relwidth=0.04, relx=0.56)
label15.place(relwidth=0.04, relx=0.60)
label16.place(relwidth=0.04, relx=0.64)
label17.place(relwidth=0.04, relx=0.68)
label18.place(relwidth=0.04, relx=0.72)
label19.place(relwidth=0.04, relx=0.76)
label20.place(relwidth=0.04, relx=0.80)
label21.place(relwidth=0.04, relx=0.84)
label22.place(relwidth=0.04, relx=0.88)
label23.place(relwidth=0.04, relx=0.92)
label24.place(relwidth=0.04, relx=0.96)

# time column labels
time1 = Label(canvas_frame, text='0800:0815', bg='#FDFEFE', relief=RIDGE)
time2 = Label(canvas_frame, text='0815:0830', bg='#FDFEFE', relief=RIDGE)
time3 = Label(canvas_frame, text='0830:0845', bg='#FDFEFE', relief=RIDGE)
time4 = Label(canvas_frame, text='0845:0900', bg='#FDFEFE', relief=RIDGE)
time5 = Label(canvas_frame, text='0900:0915', bg='#FDFEFE', relief=RIDGE)
time6 = Label(canvas_frame, text='0915:0930', bg='#FDFEFE', relief=RIDGE)
time7 = Label(canvas_frame, text='0930:0945', bg='#FDFEFE', relief=RIDGE)
time8 = Label(canvas_frame, text='0945:1000', bg='#FDFEFE', relief=RIDGE)
time9 = Label(canvas_frame, text='1000:1015', bg='#FDFEFE', relief=RIDGE)
time10 = Label(canvas_frame, text='1015:1030', bg='#FDFEFE', relief=RIDGE)
time11 = Label(canvas_frame, text='1030:1045', bg='#FDFEFE', relief=RIDGE)
time12 = Label(canvas_frame, text='1045:1100', bg='#FDFEFE', relief=RIDGE)
time13 = Label(canvas_frame, text='1100:1115', bg='#FDFEFE', relief=RIDGE)
time14 = Label(canvas_frame, text='1115:1130', bg='#FDFEFE', relief=RIDGE)
time15 = Label(canvas_frame, text='1130:1145', bg='#FDFEFE', relief=RIDGE)
time16 = Label(canvas_frame, text='1145:1200', bg='#FDFEFE', relief=RIDGE)
time17 = Label(canvas_frame, text='1200:1215', bg='#FDFEFE', relief=RIDGE)
time18 = Label(canvas_frame, text='1215:1230', bg='#FDFEFE', relief=RIDGE)
time19 = Label(canvas_frame, text='1230:1245', bg='#FDFEFE', relief=RIDGE)
time20 = Label(canvas_frame, text='1245:1300', bg='#FDFEFE', relief=RIDGE)
time21 = Label(canvas_frame, text='1300:1315', bg='#FDFEFE', relief=RIDGE)
time22 = Label(canvas_frame, text='1315:1330', bg='#FDFEFE', relief=RIDGE)
time23 = Label(canvas_frame, text='1330:1345', bg='#FDFEFE', relief=RIDGE)
time24 = Label(canvas_frame, text='1345:1400', bg='#FDFEFE', relief=RIDGE)
time25 = Label(canvas_frame, text='1400:1415', bg='#FDFEFE', relief=RIDGE)
time26 = Label(canvas_frame, text='1415:1430', bg='#FDFEFE', relief=RIDGE)
time27 = Label(canvas_frame, text='1430:1445', bg='#FDFEFE', relief=RIDGE)
time28 = Label(canvas_frame, text='1445:1500', bg='#FDFEFE', relief=RIDGE)
time29 = Label(canvas_frame, text='1500:1515', bg='#FDFEFE', relief=RIDGE)
time30 = Label(canvas_frame, text='1515:1530', bg='#FDFEFE', relief=RIDGE)
time31 = Label(canvas_frame, text='1530:1545', bg='#FDFEFE', relief=RIDGE)
time32 = Label(canvas_frame, text='1545:1600', bg='#FDFEFE', relief=RIDGE)
time33 = Label(canvas_frame, text='1600:1615', bg='#FDFEFE', relief=RIDGE)
time34 = Label(canvas_frame, text='1615:1630', bg='#FDFEFE', relief=RIDGE)
time35 = Label(canvas_frame, text='1630:1645', bg='#FDFEFE', relief=RIDGE)
time36 = Label(canvas_frame, text='1645:1700', bg='#FDFEFE', relief=RIDGE)
time37 = Label(canvas_frame, text='1700:1715', bg='#FDFEFE', relief=RIDGE)
time38 = Label(canvas_frame, text='1715:1730', bg='#FDFEFE', relief=RIDGE)
time39 = Label(canvas_frame, text='1730:1745', bg='#FDFEFE', relief=RIDGE)
time40 = Label(canvas_frame, text='1745:1800', bg='#FDFEFE', relief=RIDGE)
time41 = Label(canvas_frame, text='1800:1815', bg='#FDFEFE', relief=RIDGE)
time42 = Label(canvas_frame, text='1815:1830', bg='#FDFEFE', relief=RIDGE)
time43 = Label(canvas_frame, text='1830:1845', bg='#FDFEFE', relief=RIDGE)
time44 = Label(canvas_frame, text='1845:1900', bg='#FDFEFE', relief=RIDGE)

# placing all title for the time column
for i in range(1, 45):
    vars()['time' + str(i)].place(relwidth=0.08, y=20 + (i - 1) * 20, )

# create time-table using labels (44rows*23columns)
rows = 44
cols = 24
blk = [[0 for i in range(cols)] for j in range(rows)]
for i in range(rows):  # Rows
    for j in range(cols):  # Columns
        blk[i][j] = Label(canvas_frame, text="", bg='#FDFEFE', relief=RIDGE)
        blk[i][j].place(relwidth=0.04, relx=0.08 + j * 0.04, y=20 + i * 20)


canvas.configure(scrollregion = canvas.bbox("all"))

root.mainloop()


import datetime
from tkinter import *


def create_table(soln_list, sched_dates):

    ot_list = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'M1', 'M2', 'M3', 'M4', 'M5', 'OT 24',
               'OT 25', 'OT 22', 'R1', 'R4', 'R5', 'R6', 'R7', 'R8', 'MRI']

    discipline_codes = {10: 'BREAST',
                        11: 'OTO',
                        12: 'NES',
                        13: 'CLR',
                        14: 'HPB',
                        15: 'PLS',
                        16: 'ENT',
                        17: 'SUR-ONCO',
                        18: 'UGI',
                        19: 'H&N',
                        20: 'O&G',
                        21: 'HND',
                        22: 'OMS',
                        23: 'VAS',
                        24: 'CTS',
                        25: 'URO'
                        }

    proc_codes_breast = {10: 'mastectomy',
                         11: 'lumpectomy',
                         12: 'mastopexy',
                         13: 'biopsy',
                         14: 'mammoplasty',
                         15: 'abscess'
                         }

    proc_codes_nes = {10: 'Circumcision',
                      11: 'Pyeloplasty',
                      12: 'Nephrectomy',
                      13: 'Cystoscopy',
                      14: 'Meatoplasty',
                      15: 'Scrotoplasty'
                      }

    proc_codes_oto = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_clr = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_hpb = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_pls = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_ent = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_suronco = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_ugi = {10: 'hip',
                          11: 'tail',
                          12: 'ankle',
                          13: 'knee',
                          14: 'leg',
                          15: 'elbow'
                          }

    proc_codes_hnn = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_og = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_hnd = {10: 'hip',
                     11: 'tail',
                     12: 'ankle',
                     13: 'knee',
                     14: 'leg',
                     15: 'elbow'
                     }

    proc_codes_oms = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_vas = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_cts = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proc_codes_uro = {10: 'hip',
                      11: 'tail',
                      12: 'ankle',
                      13: 'knee',
                      14: 'leg',
                      15: 'elbow'
                      }

    proced_name_dict = { 10: proc_codes_breast,
                            11: proc_codes_oto,
                            12: proc_codes_nes,
                            13: proc_codes_clr,
                            14: proc_codes_hpb,
                            15: proc_codes_pls,
                            16: proc_codes_ent,
                            17: proc_codes_suronco,
                            18: proc_codes_ugi,
                            19: proc_codes_hnn,
                            20: proc_codes_og,
                            21: proc_codes_hnd,
                            22: proc_codes_oms,
                            23: proc_codes_vas,
                            24: proc_codes_cts,
                            25: proc_codes_uro }

    # list to collect data of actual surgeries in the 'soln' array
    # data to retrieve from array : discipline(3-4 digits) (done), procedure(5-6 digits) (done),
    # data to retrieve from array: duplicate no.(7th or last dig) (done),
    # data to retrieve from array : dates, time start and end (done), duration (done)
    actual_surgery_proced = []
    actual_surgery_durations = []
    actual_surgery_time = []
    actual_surgery_dates = []
    actual_surgery_ot = []

    final_list = []

    # sieve through each time slot in the solution list and find all actual surgeries and their allocated timeslots
    for day in range(len(soln_list)):
        for ot in range(len(soln_list[day])):
            dur_count = 0
            for time_slot in range(len(soln_list[day][ot])):
                if soln_list[day][ot][time_slot] != 0 and str(soln_list[day][ot][time_slot])[4:6] != '00' and str(soln_list[day][ot][time_slot])[1] == '1':
                    dur_count = dur_count + 1
                    # this if statement decides when to end time_slot counts (2 different criterias)
                    if (time_slot == 43 and dur_count != 0) or (str(soln_list[day][ot][time_slot]) != str(soln_list[day][ot][time_slot+1]) and dur_count != 0):
                        disc_num = int(str(soln_list[day][ot][time_slot])[2:4])
                        proced_num = int(str(soln_list[day][ot][time_slot])[4:6])
                        duplicate_num = int(str(soln_list[day][ot][time_slot])[-1])
                        duration_num = dur_count/4.0
                        time_end_deci = 8 + (time_slot+1)/4
                        time_start_deci = time_end_deci - duration_num
                        time_end = str(datetime.timedelta(hours=time_end_deci)).rsplit(':', 1)[0]
                        time_start = str(datetime.timedelta(hours=time_start_deci)).rsplit(':', 1)[0]
                        actual_surgery_ot.append(ot_list[ot])
                        actual_surgery_dates.append(sched_dates[day])
                        actual_surgery_time.append(str(time_start) + ' - ' + str(time_end))
                        actual_surgery_durations.append(str(duration_num)+' hrs')
                        actual_surgery_proced.append(proced_name_dict[disc_num][proced_num] + '_' + str(duplicate_num))
                        dur_count = 0
                else:
                    # if next timeslot is = 0, signify end of actual surgery at previous slot
                    if dur_count != 0:
                        disc_num = int(str(soln_list[day][ot][time_slot-1])[2:4])
                        proced_num = int(str(soln_list[day][ot][time_slot-1])[4:6])
                        duplicate_num = int(str(soln_list[day][ot][time_slot-1])[-1])
                        duration_num = dur_count / 4.0
                        time_end_deci = 8 + (time_slot)/4
                        time_start_deci = time_end_deci - duration_num
                        time_end = str(datetime.timedelta(hours=time_end_deci)).rsplit(':', 1)[0]
                        time_start = str(datetime.timedelta(hours=time_start_deci)).rsplit(':', 1)[0]
                        actual_surgery_ot.append(ot_list[ot])
                        actual_surgery_dates.append(sched_dates[day])
                        actual_surgery_time.append(str(time_start) + ' - ' + str(time_end))
                        actual_surgery_durations.append(str(duration_num)+' hrs')
                        actual_surgery_proced.append(proced_name_dict[disc_num][proced_num] + '_' + str(duplicate_num))
                        dur_count = 0

    actual_surgery_proced = ['Procedures :'] + actual_surgery_proced
    actual_surgery_dates = ['Dates :'] + actual_surgery_dates
    actual_surgery_ot = ['OT :'] + actual_surgery_ot
    actual_surgery_time = ['Time :'] + actual_surgery_time
    actual_surgery_durations = ['Duration :'] + actual_surgery_durations

    final_list.append(actual_surgery_proced)
    final_list.append(actual_surgery_dates)
    final_list.append(actual_surgery_ot)
    final_list.append(actual_surgery_time)
    final_list.append(actual_surgery_durations)

    rows = []
    table_screen = Tk()
    table_screen.title('List of surgery and allocated slots')
    table_screen.geometry("+750+350")

    for i in range(len(actual_surgery_time)):
        cols = []
        for j in range(5):
            e = Label(table_screen, relief=GROOVE, width=20)
            e.grid(row=i, column=j, sticky=NSEW)
            e.config(text='%s' % final_list[j][i])
            cols.append(e)
        rows.append(cols)

    table_screen.mainloop()










import numpy as np
import swapper as sp
import copy
from MO_fitting_func_calc import MO_calculate
import random

def all_legal_swaps_subset(solution, to_schedule, MAS_allowed, day_of_week_indexes, OT_indexes):
	legal_swaps = []
	# to find legal swaps between inserted surgeries and empty
	zero_indexes = zero_runs_3d(solution)

	for index, surg in enumerate(to_schedule):

		surg_loc = np.argwhere(solution == surg[0])  # this is where the surgery is slotted

		for empty_slot in zero_indexes:

			days_away_emp = empty_slot[0]
			OT_emp = empty_slot[1]
			start_index_emp = empty_slot[2]
			end_index_emp = empty_slot[3]
			dur_emp = end_index_emp - start_index_emp
			empty_slot_day = day_of_week_indexes[days_away_emp]

			days_away_surg = surg_loc[0, 0]
			OT_surg = surg_loc[0, 1]
			start_index_surg = surg_loc[0, -1]
			end_index_surg = surg_loc[-1, -1]
			if dur_emp >= len(surg) and OT_indexes[OT_emp] in MAS_allowed[empty_slot_day]:  # add MAS constraint later
				legal_swaps.append(((surg[0], days_away_surg, OT_surg, start_index_surg, end_index_surg + 1),
				                    (0, days_away_emp, OT_emp, start_index_emp, end_index_emp + 1)))
	# to find swaps between 2 inserted surgeries
	for i, surg1 in enumerate(to_schedule):
		surg1_loc = np.argwhere(solution == surg1[0])
		days_away1 = surg1_loc[0, 0]
		OT1 = surg1_loc[0, 1]
		start_index1 = surg1_loc[0, -1]
		end_index1 = surg1_loc[-1, -1]
		dur1 = end_index1 - start_index1
		dur1withempty = dur1 + find_empty_ahead(solution, days_away1, OT1, end_index1)
		surg1_slot_day = day_of_week_indexes[days_away1]
		for j, surg2 in enumerate(to_schedule):
			if i < j:
				surg2_loc = np.argwhere(solution == surg2[0])
				days_away2 = surg2_loc[0, 0]
				OT2 = surg2_loc[0, 1]
				start_index2 = surg2_loc[0, -1]
				end_index2 = surg2_loc[-1, -1]
				dur2 = end_index2 - start_index2
				dur2withempty = dur2 + find_empty_ahead(solution, days_away2, OT2, end_index2)
				surg2_slot_day = day_of_week_indexes[days_away2]
				if dur1withempty >= dur2 and dur2withempty >= dur1:  # dont need check MAS because they are both slots with inserted surgeries, which means they must agree to MAS beforehand
					legal_swaps.append(((surg1[0], days_away1, OT1, start_index1, end_index1 + 1),
					                    (surg2[0], days_away2, OT2, start_index2, end_index2 + 1)))

	return random.sample(legal_swaps,50) # will increase speed of the calculation, at the cost of quality of solution


def all_legal_swaps_with_score_calculation(solution, to_schedule, MAS_allowed, day_of_week_indexes, OT_indexes):
	legal_swaps = []
	# to find legal swaps between inserted surgeries and empty
	zero_indexes = zero_runs_3d(solution)
	scores = []

	for index, surg in enumerate(to_schedule):

		surg_loc = np.argwhere(solution == surg[0])  # this is where the surgery is slotted

		for empty_slot in zero_indexes:

			days_away_emp = empty_slot[0]
			OT_emp = empty_slot[1]
			start_index_emp = empty_slot[2]
			end_index_emp = empty_slot[3]
			dur_emp = end_index_emp - start_index_emp
			empty_slot_day = day_of_week_indexes[days_away_emp]

			days_away_surg = surg_loc[0, 0]
			OT_surg = surg_loc[0, 1]
			start_index_surg = surg_loc[0, -1]
			end_index_surg = surg_loc[-1, -1]
			if dur_emp >= len(surg) and OT_indexes[OT_emp] in MAS_allowed[empty_slot_day]:  # add MAS constraint later
				legal_swaps.append(((surg[0], days_away_surg, OT_surg, start_index_surg, end_index_surg + 1),
				                    (0, days_away_emp, OT_emp, start_index_emp, end_index_emp + 1)))
				soln = copy.deepcopy(solution)
				new = sp.swapping(soln, ((surg[0], days_away_surg, OT_surg, start_index_surg, end_index_surg + 1),
				                    (0, days_away_emp, OT_emp, start_index_emp, end_index_emp + 1)))
				scores.append(MO_calculate(new))
	# to find swaps between 2 inserted surgeries
	for i, surg1 in enumerate(to_schedule):
		surg1_loc = np.argwhere(solution == surg1[0])
		days_away1 = surg1_loc[0, 0]
		OT1 = surg1_loc[0, 1]
		start_index1 = surg1_loc[0, -1]
		end_index1 = surg1_loc[-1, -1]
		dur1 = end_index1 - start_index1
		dur1withempty = dur1 + find_empty_ahead(solution, days_away1, OT1, end_index1)
		surg1_slot_day = day_of_week_indexes[days_away1]
		for j, surg2 in enumerate(to_schedule):
			if i < j:
				surg2_loc = np.argwhere(solution == surg2[0])
				days_away2 = surg2_loc[0, 0]
				OT2 = surg2_loc[0, 1]
				start_index2 = surg2_loc[0, -1]
				end_index2 = surg2_loc[-1, -1]
				dur2 = end_index2 - start_index2
				dur2withempty = dur2 + find_empty_ahead(solution, days_away2, OT2, end_index2)
				surg2_slot_day = day_of_week_indexes[days_away2]
				if dur1withempty >= dur2 and dur2withempty >= dur1:  # dont need check MAS because they are both slots with inserted surgeries, which means they must agree to MAS beforehand
					legal_swaps.append(((surg1[0], days_away1, OT1, start_index1, end_index1 + 1),
					                    (surg2[0], days_away2, OT2, start_index2, end_index2 + 1)))
					soln = copy.deepcopy(solution)
					new = sp.swapping(soln, ((surg1[0], days_away1, OT1, start_index1, end_index1 + 1),
					                    (surg2[0], days_away2, OT2, start_index2, end_index2 + 1)))
					scores.append(MO_calculate(new))
	return legal_swaps, np.array(scores)

def all_legal_swaps(solution, to_schedule, MAS_allowed, day_of_week_indexes, OT_indexes):
	legal_swaps = []
	# to find legal swaps between inserted surgeries and empty
	zero_indexes = zero_runs_3d(solution)

	for index, surg in enumerate(to_schedule):

		surg_loc = np.argwhere(solution == surg[0])  # this is where the surgery is slotted

		for empty_slot in zero_indexes:

			days_away_emp = empty_slot[0]
			OT_emp = empty_slot[1]
			start_index_emp = empty_slot[2]
			end_index_emp = empty_slot[3]
			dur_emp = end_index_emp - start_index_emp
			empty_slot_day = day_of_week_indexes[days_away_emp]

			days_away_surg = surg_loc[0, 0]
			OT_surg = surg_loc[0, 1]
			start_index_surg = surg_loc[0, -1]
			end_index_surg = surg_loc[-1, -1]
			if dur_emp >= len(surg) and OT_indexes[OT_emp] in MAS_allowed[empty_slot_day]:  # add MAS constraint later
				legal_swaps.append(((surg[0], days_away_surg, OT_surg, start_index_surg, end_index_surg + 1), (0, days_away_emp, OT_emp, start_index_emp, end_index_emp + 1)))
	# to find swaps between 2 inserted surgeries
	for i, surg1 in enumerate(to_schedule):
		surg1_loc = np.argwhere(solution == surg1[0])
		days_away1 = surg1_loc[0,0]
		OT1 = surg1_loc[0, 1]
		start_index1 = surg1_loc[0, -1]
		end_index1 = surg1_loc[-1, -1]
		dur1 = end_index1 - start_index1
		dur1withempty = dur1 + find_empty_ahead(solution, days_away1, OT1, end_index1)
		surg1_slot_day = day_of_week_indexes[days_away1]
		for j, surg2 in enumerate(to_schedule):
			if i < j:
				surg2_loc = np.argwhere(solution == surg2[0])
				days_away2 = surg2_loc[0, 0]
				OT2 = surg2_loc[0, 1]
				start_index2 = surg2_loc[0, -1]
				end_index2 = surg2_loc[-1, -1]
				dur2 = end_index2 - start_index2
				dur2withempty = dur2 + find_empty_ahead(solution, days_away2, OT2, end_index2)
				surg2_slot_day = day_of_week_indexes[days_away2]
				if dur1withempty >= dur2 and dur2withempty >= dur1: # dont need check MAS because they are both slots with inserted surgeries, which means they must agree to MAS beforehand
					legal_swaps.append(((surg1[0], days_away1, OT1, start_index1, end_index1 + 1),(surg2[0],days_away2, OT2, start_index2, end_index2 + 1)))

	return legal_swaps


def zero_runs_3d(a):
	iszero = np.equal(a, 0)
	npad = ((0, 0), (0, 0), (1, 1))
	iszero = np.pad(iszero, pad_width=npad, mode='constant', constant_values=0).view(np.int8)
	absdiff = np.abs(np.diff(iszero, axis=2))
	ranges = np.argwhere(absdiff == 1)
	even = ranges[::2]
	odd_c = ranges[1::2][:, 2:]
	return np.hstack((even, odd_c))



def find_empty_ahead(soln, days_away, OT, end):
	block = soln[days_away][OT]
	i = end
	extrazero = 0
	while block[i] == 0:
		extrazero += 1
		i += 1
	return extrazero


def zero_runs(a):
	# Create an array that is 1 where a is 0, and pad each end with an extra 0.
	iszero = np.concatenate(([0], np.equal(a, 0).view(np.int8), [0]))
	absdiff = np.abs(np.diff(iszero))
	# Runs start and end where absdiff is 1.
	r = np.where(absdiff==1)[0]
	ranges = np.where(absdiff == 1)[0].reshape(-1, 2)
	lendiff = np.diff(ranges, axis=1)
	return ranges.tolist(), lendiff.tolist()


import os
from tkinter import *
from tkinter import filedialog
import csv
from datetime import *


def read_mas():

    mas_allowed = {'BREAST': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                              'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                              'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                              'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                              'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'HND': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'SUR-ONCO': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                                'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                                'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                                'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                                'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'HPB': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'OTO': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'H&N': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'CLR': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'ENT': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'NES': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'VAS': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'O&G': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'UGI': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'PLS': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'CTS': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'URO': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'OMS': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []}}

    # this is a window prompt for selecting csv file to be read
    dialog_root = Tk()
    dialog_root.geometry("1200x700+350+150")
    dialog_root.withdraw()

    # this will obtain file path of the selected csv file
    file_path = filedialog.askopenfilename(title='Please select the MAS schedule .csv file')
    file_path = os.path.normpath(file_path)

    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        next(mock_csv, None)  # skip header
        for row in mock_csv:  # get unique dates in the csv file
            if '/' not in row[4]:  # if discipline column does not contain '/'
                if row[4] in mas_allowed and row[1] == str(0):
                    mas_allowed[row[4]]['Monday'+str(row[3])].append(row[0])
                if row[4] in mas_allowed and row[1] == str(1):
                    mas_allowed[row[4]]['Tuesday'+str(row[3])].append(row[0])
                if row[4] in mas_allowed and row[1] == str(2):
                    mas_allowed[row[4]]['Wednesday'+str(row[3])].append(row[0])
                if row[4] in mas_allowed and row[1] == str(3):
                    mas_allowed[row[4]]['Thursday'+str(row[3])].append(row[0])
                if row[4] in mas_allowed and row[1] == str(4):
                    mas_allowed[row[4]]['Friday'+str(row[3])].append(row[0])
            if '/' in row[4]:  # if discipline column contains '/'
                disciplines_names = row[4].split('/')
                for discs in disciplines_names:
                    if discs in mas_allowed and row[1] == str(0):
                        mas_allowed[discs]['Monday' + str(row[3])].append(row[0])
                    if discs in mas_allowed and row[1] == str(1):
                        mas_allowed[discs]['Tuesday' + str(row[3])].append(row[0])
                    if discs in mas_allowed and row[1] == str(2):
                        mas_allowed[discs]['Wednesday' + str(row[3])].append(row[0])
                    if discs in mas_allowed and row[1] == str(3):
                        mas_allowed[discs]['Thursday' + str(row[3])].append(row[0])
                    if discs in mas_allowed and row[1] == str(4):
                        mas_allowed[discs]['Friday' + str(row[3])].append(row[0])

    return mas_allowed


def read_mas1(file_path):
    mas_allowed = {'BREAST': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                              'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                              'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                              'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                              'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'HND': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'SUR-ONCO': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                                'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                                'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                                'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                                'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'HPB': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'OTO': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'H&N': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'CLR': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'ENT': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'NES': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'VAS': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'O&G': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'UGI': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'PLS': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'CTS': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'URO': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []},
                   'OMS': {'Monday1': [], 'Tuesday1': [], 'Wednesday1': [], 'Thursday1': [], 'Friday1': [],
                           'Monday2': [], 'Tuesday2': [], 'Wednesday2': [], 'Thursday2': [], 'Friday2': [],
                           'Monday3': [], 'Tuesday3': [], 'Wednesday3': [], 'Thursday3': [], 'Friday3': [],
                           'Monday4': [], 'Tuesday4': [], 'Wednesday4': [], 'Thursday4': [], 'Friday4': [],
                           'Monday5': [], 'Tuesday5': [], 'Wednesday5': [], 'Thursday5': [], 'Friday5': []}}

    # this is a window prompt for selecting csv file to be read
    dialog_root = Tk()
    dialog_root.geometry("1200x700+350+150")
    dialog_root.withdraw()

    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        next(mock_csv, None)  # skip header
        for row in mock_csv:  # get unique dates in the csv file
            if '/' not in row[4]:  # if discipline column does not contain '/'
                if row[4] in mas_allowed and row[1] == str(0):
                    mas_allowed[row[4]]['Monday' + str(row[3])].append(row[0])
                if row[4] in mas_allowed and row[1] == str(1):
                    mas_allowed[row[4]]['Tuesday' + str(row[3])].append(row[0])
                if row[4] in mas_allowed and row[1] == str(2):
                    mas_allowed[row[4]]['Wednesday' + str(row[3])].append(row[0])
                if row[4] in mas_allowed and row[1] == str(3):
                    mas_allowed[row[4]]['Thursday' + str(row[3])].append(row[0])
                if row[4] in mas_allowed and row[1] == str(4):
                    mas_allowed[row[4]]['Friday' + str(row[3])].append(row[0])
            if '/' in row[4]:  # if discipline column contains '/'
                disciplines_names = row[4].split('/')
                for discs in disciplines_names:
                    if discs in mas_allowed and row[1] == str(0):
                        mas_allowed[discs]['Monday' + str(row[3])].append(row[0])
                    if discs in mas_allowed and row[1] == str(1):
                        mas_allowed[discs]['Tuesday' + str(row[3])].append(row[0])
                    if discs in mas_allowed and row[1] == str(2):
                        mas_allowed[discs]['Wednesday' + str(row[3])].append(row[0])
                    if discs in mas_allowed and row[1] == str(3):
                        mas_allowed[discs]['Thursday' + str(row[3])].append(row[0])
                    if discs in mas_allowed and row[1] == str(4):
                        mas_allowed[discs]['Friday' + str(row[3])].append(row[0])

    return mas_allowed


from tkinter import *
from tkinter import ttk
from tkcalendar import *
from datetime import *


def show_timetable(current_schedule):

    root = Tk()

    # set title and size of the GUI window
    root.title('Timetable')
    root.geometry("1200x700+350+150")

    # set secondary frame position and size (center right)
    frame = Frame(root)
    frame.place(relx=0.05, rely=0.05, relwidth=0.9, relheight=0.9)

    # get today's date and set as heading
    today = date.today()
    if today.weekday() == 5:
        today = date.today() - timedelta(days=1)
    if today.weekday() == 6:
        today = date.today() - timedelta(days=2)
    week_day = today.strftime("%A")

    heading = Label(frame, text= week_day + " , " + str(today), bg='#E5E7E9', relief=RIDGE)
    heading.place(relx=0.38, relwidth=0.25, y=0)

    # set secondary frame position and size
    canvas = Canvas(root, bg='#F4F6F7')
    canvas.place(relx=0.1, rely=0.1, relwidth=0.8, relheight=0.8)

    vbar = Scrollbar(frame)
    vbar.config(command=canvas.yview)
    canvas.config(yscrollcommand=vbar.set)
    vbar.pack(side="right", fill=Y)

    # create a frame inside the canvas
    canvas_frame = Frame(canvas, width=960, height=1000)
    canvas_frame.pack(expand=True, fill=BOTH)

    canvas.create_window(0, 0, window=canvas_frame, anchor="nw")

    # time-table headings
    label1 = Label(canvas_frame, text='Time:', bg='#D0ECE7', relief=RIDGE)
    label2 = Label(canvas_frame, text='L1', bg='#D0ECE7', relief=RIDGE)
    label3 = Label(canvas_frame, text='L2', bg='#D0ECE7', relief=RIDGE)
    label4 = Label(canvas_frame, text='L3', bg='#D0ECE7', relief=RIDGE)
    label5 = Label(canvas_frame, text='L4', bg='#D0ECE7', relief=RIDGE)
    label6 = Label(canvas_frame, text='L5', bg='#D0ECE7', relief=RIDGE)
    label7 = Label(canvas_frame, text='L6', bg='#D0ECE7', relief=RIDGE)
    label8 = Label(canvas_frame, text='L7', bg='#D0ECE7', relief=RIDGE)
    label9 = Label(canvas_frame, text='L8', bg='#D0ECE7', relief=RIDGE)
    label10 = Label(canvas_frame, text='M1', bg='#D0ECE7', relief=RIDGE)
    label11 = Label(canvas_frame, text='M2', bg='#D0ECE7', relief=RIDGE)
    label12 = Label(canvas_frame, text='M3', bg='#D0ECE7', relief=RIDGE)
    label13 = Label(canvas_frame, text='M4', bg='#D0ECE7', relief=RIDGE)
    label14 = Label(canvas_frame, text='M5', bg='#D0ECE7', relief=RIDGE)
    label15 = Label(canvas_frame, text='OT 24', bg='#D0ECE7', relief=RIDGE)
    label16 = Label(canvas_frame, text='OT 25', bg='#D0ECE7', relief=RIDGE)
    label17 = Label(canvas_frame, text='OT 22', bg='#D0ECE7', relief=RIDGE)
    label18 = Label(canvas_frame, text='R1', bg='#D0ECE7', relief=RIDGE)
    label19 = Label(canvas_frame, text='R4', bg='#D0ECE7', relief=RIDGE)
    label20 = Label(canvas_frame, text='R5', bg='#D0ECE7', relief=RIDGE)
    label21 = Label(canvas_frame, text='R6', bg='#D0ECE7', relief=RIDGE)
    label22 = Label(canvas_frame, text='R7', bg='#D0ECE7', relief=RIDGE)
    label23 = Label(canvas_frame, text='R8', bg='#D0ECE7', relief=RIDGE)
    label24 = Label(canvas_frame, text='MRI', bg='#D0ECE7', relief=RIDGE)
    label1.place(relwidth=0.08)
    label2.place(relwidth=0.04, relx=0.08)
    label3.place(relwidth=0.04, relx=0.12)
    label4.place(relwidth=0.04, relx=0.16)
    label5.place(relwidth=0.04, relx=0.20)
    label6.place(relwidth=0.04, relx=0.24)
    label7.place(relwidth=0.04, relx=0.28)
    label8.place(relwidth=0.04, relx=0.32)
    label9.place(relwidth=0.04, relx=0.36)
    label10.place(relwidth=0.04, relx=0.40)
    label11.place(relwidth=0.04, relx=0.44)
    label12.place(relwidth=0.04, relx=0.48)
    label13.place(relwidth=0.04, relx=0.52)
    label14.place(relwidth=0.04, relx=0.56)
    label15.place(relwidth=0.04, relx=0.60)
    label16.place(relwidth=0.04, relx=0.64)
    label17.place(relwidth=0.04, relx=0.68)
    label18.place(relwidth=0.04, relx=0.72)
    label19.place(relwidth=0.04, relx=0.76)
    label20.place(relwidth=0.04, relx=0.80)
    label21.place(relwidth=0.04, relx=0.84)
    label22.place(relwidth=0.04, relx=0.88)
    label23.place(relwidth=0.04, relx=0.92)
    label24.place(relwidth=0.04, relx=0.96)

    # time column labels
    time1 = Label(canvas_frame, text='0800:0815', bg='#FDFEFE', relief=RIDGE)
    time2 = Label(canvas_frame, text='0815:0830', bg='#FDFEFE', relief=RIDGE)
    time3 = Label(canvas_frame, text='0830:0845', bg='#FDFEFE', relief=RIDGE)
    time4 = Label(canvas_frame, text='0845:0900', bg='#FDFEFE', relief=RIDGE)
    time5 = Label(canvas_frame, text='0900:0915', bg='#FDFEFE', relief=RIDGE)
    time6 = Label(canvas_frame, text='0915:0930', bg='#FDFEFE', relief=RIDGE)
    time7 = Label(canvas_frame, text='0930:0945', bg='#FDFEFE', relief=RIDGE)
    time8 = Label(canvas_frame, text='0945:1000', bg='#FDFEFE', relief=RIDGE)
    time9 = Label(canvas_frame, text='1000:1015', bg='#FDFEFE', relief=RIDGE)
    time10 = Label(canvas_frame, text='1015:1030', bg='#FDFEFE', relief=RIDGE)
    time11 = Label(canvas_frame, text='1030:1045', bg='#FDFEFE', relief=RIDGE)
    time12 = Label(canvas_frame, text='1045:1100', bg='#FDFEFE', relief=RIDGE)
    time13 = Label(canvas_frame, text='1100:1115', bg='#FDFEFE', relief=RIDGE)
    time14 = Label(canvas_frame, text='1115:1130', bg='#FDFEFE', relief=RIDGE)
    time15 = Label(canvas_frame, text='1130:1145', bg='#FDFEFE', relief=RIDGE)
    time16 = Label(canvas_frame, text='1145:1200', bg='#FDFEFE', relief=RIDGE)
    time17 = Label(canvas_frame, text='1200:1215', bg='#FDFEFE', relief=RIDGE)
    time18 = Label(canvas_frame, text='1215:1230', bg='#FDFEFE', relief=RIDGE)
    time19 = Label(canvas_frame, text='1230:1245', bg='#FDFEFE', relief=RIDGE)
    time20 = Label(canvas_frame, text='1245:1300', bg='#FDFEFE', relief=RIDGE)
    time21 = Label(canvas_frame, text='1300:1315', bg='#FDFEFE', relief=RIDGE)
    time22 = Label(canvas_frame, text='1315:1330', bg='#FDFEFE', relief=RIDGE)
    time23 = Label(canvas_frame, text='1330:1345', bg='#FDFEFE', relief=RIDGE)
    time24 = Label(canvas_frame, text='1345:1400', bg='#FDFEFE', relief=RIDGE)
    time25 = Label(canvas_frame, text='1400:1415', bg='#FDFEFE', relief=RIDGE)
    time26 = Label(canvas_frame, text='1415:1430', bg='#FDFEFE', relief=RIDGE)
    time27 = Label(canvas_frame, text='1430:1445', bg='#FDFEFE', relief=RIDGE)
    time28 = Label(canvas_frame, text='1445:1500', bg='#FDFEFE', relief=RIDGE)
    time29 = Label(canvas_frame, text='1500:1515', bg='#FDFEFE', relief=RIDGE)
    time30 = Label(canvas_frame, text='1515:1530', bg='#FDFEFE', relief=RIDGE)
    time31 = Label(canvas_frame, text='1530:1545', bg='#FDFEFE', relief=RIDGE)
    time32 = Label(canvas_frame, text='1545:1600', bg='#FDFEFE', relief=RIDGE)
    time33 = Label(canvas_frame, text='1600:1615', bg='#FDFEFE', relief=RIDGE)
    time34 = Label(canvas_frame, text='1615:1630', bg='#FDFEFE', relief=RIDGE)
    time35 = Label(canvas_frame, text='1630:1645', bg='#FDFEFE', relief=RIDGE)
    time36 = Label(canvas_frame, text='1645:1700', bg='#FDFEFE', relief=RIDGE)
    time37 = Label(canvas_frame, text='1700:1715', bg='#FDFEFE', relief=RIDGE)
    time38 = Label(canvas_frame, text='1715:1730', bg='#FDFEFE', relief=RIDGE)
    time39 = Label(canvas_frame, text='1730:1745', bg='#FDFEFE', relief=RIDGE)
    time40 = Label(canvas_frame, text='1745:1800', bg='#FDFEFE', relief=RIDGE)
    time41 = Label(canvas_frame, text='1800:1815', bg='#FDFEFE', relief=RIDGE)
    time42 = Label(canvas_frame, text='1815:1830', bg='#FDFEFE', relief=RIDGE)
    time43 = Label(canvas_frame, text='1830:1845', bg='#FDFEFE', relief=RIDGE)
    time44 = Label(canvas_frame, text='1845:1900', bg='#FDFEFE', relief=RIDGE)

    # placing all title for the time column
    for i in range(1, 45):
        vars()['time' + str(i)].place(relwidth=0.08, y=20 + (i - 1) * 20, )
        i = i + 1

    # create time-table using labels (44rows*23columns)
    rows = 44
    cols = 24
    blk = [[0 for i in range(cols)] for j in range(rows)]
    for i in range(rows):  # Rows
        for j in range(cols):  # Columns
            blk[i][j] = Label(canvas_frame, text="", bg='#FDFEFE', relief=RIDGE)
            blk[i][j].place(relwidth=0.04, relx=0.08 + j * 0.04, y=20 + i * 20)

    color_code = {'OTO':'#D35400', 'H&N':'#D7BDE2', 'CLR':'#27AE60', 'HPB':'#F4ECF7', 'SUR-ONCO':'#E91E63',
                  'BREAST':'#00FF33', 'ENT':'#FFF59D', 'NES':'#FFF59D', 'VAS':'#E0F7FA', 'O&G':'#757575',
                  'HND':'#E0E0E0', 'UGI':'#006600', 'PLS':'#F5F5F5', 'CTS':'#A569BD', 'URO':'#5DADE2',
                  'OMS':'#A9CCE3', 'Actual':'#3F51B5', 'Predicted':'#34495E'}

    ot_dict = {'L1': 0, 'L2': 1, 'L3': 2, 'L4': 3, 'L5': 4, 'L6': 5, 'L7': 6, 'L8': 7, 'M1': 8, 'M2':9, 'M3': 10, 'M4': 11, 'M5': 12, 'OT 24': 13,
               'OT 25': 14, 'OT 22': 15, 'R1': 16, 'R4': 17, 'R5': 18, 'R6': 19, 'R7': 20, 'R8': 21, 'MRI': 22}

    if 0 in current_schedule:
        for i in current_schedule[0][0]:
            for j in current_schedule[0][0][i]:
                # ini_pos = j[0]
                if j[3] != 'empty' and j[3] in color_code:
                    ini_pos = int((j[0] - 8) * 4)
                    tot_dur = int(j[2] / 0.25)
                    for k in range(tot_dur):
                        blk[ini_pos + k][ot_dict[i]].config(bg=color_code[j[3]], text=j[3])
                elif j[3] != 'empty' and j[3] not in color_code and j[5] == False:
                    ini_pos = int((j[0] - 8) * 4)
                    tot_dur = int(j[2] / 0.25)
                    for k in range(tot_dur):
                        blk[ini_pos + k][ot_dict[i]].config(bg=color_code['Predicted'], text=j[3])
                elif j[3] != 'empty' and j[3] not in color_code and j[5] == True:
                    ini_pos = int((j[0] - 8) * 4)
                    tot_dur = int(j[2] / 0.25)
                    for k in range(tot_dur):
                        blk[ini_pos + k][ot_dict[i]].config(bg=color_code['Actual'], text=j[3])
    else:
        for i in range(rows):  # Rows
            for j in range(cols):  # Columns
                blk[i][j].config(bg='white', text='')

    # defining the prev and next function
    def prev_date():
        for i in range(rows):  # Rows
            for j in range(cols):  # Columns
                blk[i][j].config(bg='white', text='')
        str_date = heading.cget('text')
        parts = str_date.split(' , ')
        date_now = datetime.strptime(parts[1], '%Y-%m-%d').date()
        date_prev = date_now - timedelta(days=1)
        if date_prev.weekday() == 6:
            date_prev = date_prev - timedelta(days=2)
        else:
            pass
        day_prev = date_prev.strftime("%A")
        heading.config(text=str(day_prev) + " , " + str(date_prev))

        days_xx = (date_prev - date.today()).days
        # grey out current scheduled appointments
        if days_xx in current_schedule:
            for i in current_schedule[days_xx][0]:
                for j in current_schedule[days_xx][0][i]:
                    # ini_pos = j[0]
                    if j[3] != 'empty' and j[3] in color_code:
                        ini_pos = int((j[0] - 8) * 4)
                        tot_dur = int(j[2] / 0.25)
                        for k in range(tot_dur):
                            blk[ini_pos + k][ot_dict[i]].config(bg=color_code[j[3]], text=j[3])
                    elif j[3] != 'empty' and j[3] not in color_code and j[5] == False:
                        ini_pos = int((j[0] - 8) * 4)
                        tot_dur = int(j[2] / 0.25)
                        for k in range(tot_dur):
                            blk[ini_pos + k][ot_dict[i]].config(bg=color_code['Predicted'], text=j[3])
                    elif j[3] != 'empty' and j[3] not in color_code and j[5] == True:
                        ini_pos = int((j[0] - 8) * 4)
                        tot_dur = int(j[2] / 0.25)
                        for k in range(tot_dur):
                            blk[ini_pos + k][ot_dict[i]].config(bg=color_code['Actual'], text=j[3])
        else:
            for i in range(rows):  # Rows
                for j in range(cols):  # Columns
                    blk[i][j].config(bg='white', text='')

    def next_date():
        for i in range(rows):  # Rows
            for j in range(cols):  # Columns
                blk[i][j].config(bg='white', text='')
        str_date = heading.cget('text')
        parts = str_date.split(' , ')
        date_now = datetime.strptime(parts[1], '%Y-%m-%d').date()
        date_next = date_now + timedelta(days=1)
        if date_next.weekday() == 5:
            date_next = date_next + timedelta(days=2)
        else:
            pass
        day_next = date_next.strftime("%A")
        heading.config(text=str(day_next) + " , " + str(date_next))
        # x = label indicated at top of GUI
        days_xx = (date_next - date.today()).days
        # grey out current scheduled appointments
        if days_xx in current_schedule:
            for i in current_schedule[days_xx][0]:
                for j in current_schedule[days_xx][0][i]:
                    # ini_pos = j[0]
                    if j[3] != 'empty' and j[3] in color_code:
                        ini_pos = int((j[0] - 8) * 4)
                        tot_dur = int(j[2] / 0.25)
                        for k in range(tot_dur):
                            blk[ini_pos + k][ot_dict[i]].config(bg=color_code[j[3]], text=j[3])
                    elif j[3] != 'empty' and j[3] not in color_code and j[5] == False:
                        ini_pos = int((j[0] - 8) * 4)
                        tot_dur = int(j[2] / 0.25)
                        for k in range(tot_dur):
                            blk[ini_pos + k][ot_dict[i]].config(bg=color_code['Predicted'], text=j[3])
                    elif j[3] != 'empty' and j[3] not in color_code and j[5] == True:
                        ini_pos = int((j[0] - 8) * 4)
                        tot_dur = int(j[2] / 0.25)
                        for k in range(tot_dur):
                            blk[ini_pos + k][ot_dict[i]].config(bg=color_code['Actual'], text=j[3])
        else:
            for i in range(rows):  # Rows
                for j in range(cols):  # Columns
                    blk[i][j].config(bg='white', text='')

    prev_button = Button(frame, text='prev', width=10, bg='#E5E7E9',command=prev_date)
    next_button = Button(frame, text='next', width=10, bg='#E5E7E9',command=next_date)
    prev_button.place(relx=0.40, rely=0.95)
    next_button.place(relx=0.55, rely=0.95)

    canvas.configure(scrollregion = canvas.bbox("all"))

    root.mainloop()


import numpy as np
import os
import csv
from tkinter import *
from tkinter import filedialog
from datetime import *
from view_timetable_old import show_timetable

# current schedule should have 23 OTs and 44 Rows(to represent 44-15min timeslots)
# current_schedule = np.zeros((5,23,44))

# possible ways to read the data
# find number of unique dates in the csv file
# create numpy array with (23,44,n) where n corresponds to number of unique days in current schedule


def read_csv():

    # this is a window prompt for selecting csv file to be read
    dialog_root = Tk()
    dialog_root.geometry("1200x700+350+150")
    dialog_root.withdraw()

    # this will obtain file path of the selected csv file
    file_path = filedialog.askopenfilename(title='Please select the schedule .csv file')
    file_path = os.path.normpath(file_path)

    current_dates = [] # store the list of unique dates in the csv file
    temp_days = []  # this temp list will number of days from now for each unique date, 3, 4 ...
    key_list = []

    # this will find number of unique dates in the current schedule
    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        next(mock_csv, None)  # skip header
        for row in mock_csv:  # get unique dates in the csv file
            items = row[1].split(' ')
            if items[0] not in current_dates:
                current_dates.append(items[0])

    # for each day in current_dates, create a dictionary i.e day4 = {}
    for i in current_dates:
        selected_date = datetime.strptime(i, '%d/%m/%Y').date()
        days_from_now = (selected_date - date.today()).days
        temp_days.append(days_from_now)  # the days from today for each day_dictionary is stored in temp_days

    for i in range(len(temp_days)):
        key_list.append(i)

    d1 = '2020-11-30'
    d1 = datetime.strptime(d1, '%Y-%m-%d').date()
    d2 = '2020-12-01'
    d2 = datetime.strptime(d2, '%Y-%m-%d').date()
    d3 = '2020-12-02'
    d3 = datetime.strptime(d3, '%Y-%m-%d').date()
    d4 = '2020-12-03'
    d4 = datetime.strptime(d4, '%Y-%m-%d').date()
    d5 = '2020-12-04'
    d5 = datetime.strptime(d5, '%Y-%m-%d').date()

    week_dict = {0: 1, 7: 2, 14: 3, 21: 4, 28: 5}

    value_list = []

    for i in temp_days:  # this will load all the necessary days into the current_schedule dictionary
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Monday':
            temp_date = (((date.today() + timedelta(days=i))-d1).days)%35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Tuesday':
            temp_date = (((date.today() + timedelta(days=i))-d2).days)%35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Wednesday':
            temp_date = (((date.today() + timedelta(days=i))-d3).days)%35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Thursday':
            temp_date = (((date.today() + timedelta(days=i))-d4).days)%35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Friday':
            temp_date = (((date.today() + timedelta(days=i))-d5).days)%35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))

    day_of_week_index = {key_list[i]: value_list[i] for i in range(len(key_list))}

    # create current schedule numpy array filled with zeros
    current_schedule = np.zeros((len(current_dates), 23, 44), dtype=np.int32)

    # (days, OT, time)
    # 20121300 = [2] unswappable, [0] currentsurgery, [12] Discipline type 12, [13] Procedure 13, [0] = not important for unswappable surgeries
    # Everytime its on the array, means the 15 min slot is taken up
    # everything on the initial current scehdule will start with 20
    # All inserted surgeries will begin with 1 (swappable),
    # Second digit of inserted surgeries will be 1 (actual) or 2(predicted)
    # axis = 2 will always be of length 44

    # discipline dictionary to identify code
    discipline_codes = {10: 'BREAST',
                        11: 'OTO',
                        12: 'NES',
                        13: 'CLR',
                        14: 'HPB',
                        15: 'PLS',
                        16: 'ENT',
                        17: 'SUR-ONCO',
                        18: 'UGI',
                        19: 'H&N',
                        20: 'O&G',
                        21: 'HND',
                        22: 'OMS',
                        23: 'VAS',
                        24: 'CTS',
                        25: 'URO'
                        }

    key_list = list(discipline_codes.keys())
    value_list = list(discipline_codes.values())

    # ot_list in order
    ot_list = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'M1', 'M2', 'M3', 'M4', 'M5', 'OT 24',
               'OT 25', 'OT 22', 'R1', 'R4', 'R5', 'R6', 'R7', 'R8', 'MRI']

    current_schedule_list = []

    # read each line in csv and append details to the respective 'OT' dictionaries
    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        # skip header
        next(mock_csv, None)
        for row in mock_csv:
            start_date = row[1].split(' ')
            end_date = row[2].split(' ')
            start_time = start_date[1]  # start time
            end_time = end_date[1]  # end time
            # codes to convert time into appropriate format 08:15 = 8.25, 10:45 = 10.75
            start_time_new = datetime.strptime(start_time, '%H:%M').time()
            end_time_new = datetime.strptime(end_time, '%H:%M').time()
            start_time_deci = (start_time_new.hour + start_time_new.minute / 60.0)
            start_index = int((start_time_deci - 8)/0.25)
            end_time_deci = (end_time_new.hour + end_time_new.minute / 60.0)
            end_index = int((end_time_deci - 8)/0.25)
            duration = int((end_time_deci - start_time_deci)/0.25)  # calculate duration in 15min blocks
            date_index = current_dates.index(start_date[0])  # gets index of the current date
            # this will append the necessary codes into each OT array
            position = value_list.index(row[4])
            current_schedule[date_index, ot_list.index(row[5]), start_index:end_index] = 2000000 + (key_list[position])*1000

    # check current_dates and add 'temp_days' and 'current_dates'
    if (datetime.strptime(current_dates[-1], '%d/%m/%Y').date()).strftime('%A') == 'Friday':
        x1 = (datetime.strptime(current_dates[-1], '%d/%m/%Y').date())
        x2 = temp_days[-1]
        temp_array = np.zeros([1, 23, 44], dtype=np.int32)
        dict_elem_count = len(current_dates)
        mas_week_count = int(day_of_week_index[dict_elem_count - 1].split('y')[1])
        if mas_week_count != 5:
            mas_week_count = mas_week_count + 1
        else:
            mas_week_count = 1
        for i in range(3, 8):
            new_date = x1 + timedelta(days=i)
            new_datez = new_date.strftime("%d/%m/%Y")
            new_day = new_date.strftime('%A')
            current_dates.append(new_datez)
            new_dayfromnow = x2 + i
            temp_days.append(new_dayfromnow)
            current_schedule = np.concatenate([current_schedule, temp_array], axis=0)
            day_of_week_index[dict_elem_count] = str(new_day) + str(mas_week_count)
            dict_elem_count = dict_elem_count + 1

    current_schedule_list.append(current_schedule)
    current_schedule_list.append(temp_days)
    current_schedule_list.append(current_dates)
    current_schedule_list.append(day_of_week_index)

    return current_schedule_list


def read_csv1(file_path):
    # this is a window prompt for selecting csv file to be read
    dialog_root = Tk()
    dialog_root.geometry("1200x700+350+150")
    dialog_root.withdraw()

    current_dates = []  # store the list of unique dates in the csv file
    temp_days = []  # this temp list will number of days from now for each unique date, 3, 4 ...
    key_list = []

    # this will find number of unique dates in the current schedule
    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        next(mock_csv, None)  # skip header
        for row in mock_csv:  # get unique dates in the csv file
            items = row[1].split(' ')
            if items[0] not in current_dates:
                current_dates.append(items[0])

    # for each day in current_dates, create a dictionary i.e day4 = {}
    for i in current_dates:
        selected_date = datetime.strptime(i, '%d/%m/%Y').date()
        days_from_now = (selected_date - date.today()).days
        temp_days.append(days_from_now)  # the days from today for each day_dictionary is stored in temp_days

    for i in range(len(temp_days)):
        key_list.append(i)

    d1 = '2020-11-30'
    d1 = datetime.strptime(d1, '%Y-%m-%d').date()
    d2 = '2020-12-01'
    d2 = datetime.strptime(d2, '%Y-%m-%d').date()
    d3 = '2020-12-02'
    d3 = datetime.strptime(d3, '%Y-%m-%d').date()
    d4 = '2020-12-03'
    d4 = datetime.strptime(d4, '%Y-%m-%d').date()
    d5 = '2020-12-04'
    d5 = datetime.strptime(d5, '%Y-%m-%d').date()

    week_dict = {0: 1, 7: 2, 14: 3, 21: 4, 28: 5}

    value_list = []

    for i in temp_days:  # this will load all the necessary days into the current_schedule dictionary
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Monday':
            temp_date = (((date.today() + timedelta(days=i)) - d1).days) % 35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Tuesday':
            temp_date = (((date.today() + timedelta(days=i)) - d2).days) % 35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Wednesday':
            temp_date = (((date.today() + timedelta(days=i)) - d3).days) % 35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Thursday':
            temp_date = (((date.today() + timedelta(days=i)) - d4).days) % 35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))
        if (datetime.today() + timedelta(days=i)).strftime('%A') == 'Friday':
            temp_date = (((date.today() + timedelta(days=i)) - d5).days) % 35
            value_list.append((date.today() + timedelta(days=i)).strftime('%A') + str(week_dict[temp_date]))

    day_of_week_index = {key_list[i]: value_list[i] for i in range(len(key_list))}

    # create current schedule numpy array filled with zeros
    current_schedule = np.zeros((len(current_dates), 23, 44), dtype=np.int32)

    # (days, OT, time)
    # 20121300 = [2] unswappable, [0] currentsurgery, [12] Discipline type 12, [13] Procedure 13, [0] = not important for unswappable surgeries
    # Everytime its on the array, means the 15 min slot is taken up
    # everything on the initial current scehdule will start with 20
    # All inserted surgeries will begin with 1 (swappable),
    # Second digit of inserted surgeries will be 1 (actual) or 2(predicted)
    # axis = 2 will always be of length 44

    # discipline dictionary to identify code
    discipline_codes = {10: 'BREAST',
                        11: 'OTO',
                        12: 'NES',
                        13: 'CLR',
                        14: 'HPB',
                        15: 'PLS',
                        16: 'ENT',
                        17: 'SUR-ONCO',
                        18: 'UGI',
                        19: 'H&N',
                        20: 'O&G',
                        21: 'HND',
                        22: 'OMS',
                        23: 'VAS',
                        24: 'CTS',
                        25: 'URO'
                        }

    key_list = list(discipline_codes.keys())
    value_list = list(discipline_codes.values())

    # ot_list in order
    ot_list = ['L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8', 'M1', 'M2', 'M3', 'M4', 'M5', 'OT 24',
               'OT 25', 'OT 22', 'R1', 'R4', 'R5', 'R6', 'R7', 'R8', 'MRI']

    current_schedule_list = []

    # read each line in csv and append details to the respective 'OT' dictionaries
    with open(file_path, 'r', newline='') as csv_file:
        mock_csv = csv.reader(csv_file, delimiter=',')
        # skip header
        next(mock_csv, None)
        for row in mock_csv:
            start_date = row[1].split(' ')
            end_date = row[2].split(' ')
            start_time = start_date[1]  # start time
            end_time = end_date[1]  # end time
            # codes to convert time into appropriate format 08:15 = 8.25, 10:45 = 10.75
            start_time_new = datetime.strptime(start_time, '%H:%M').time()
            end_time_new = datetime.strptime(end_time, '%H:%M').time()
            start_time_deci = (start_time_new.hour + start_time_new.minute / 60.0)
            start_index = int((start_time_deci - 8) / 0.25)
            end_time_deci = (end_time_new.hour + end_time_new.minute / 60.0)
            end_index = int((end_time_deci - 8) / 0.25)
            duration = int((end_time_deci - start_time_deci) / 0.25)  # calculate duration in 15min blocks
            date_index = current_dates.index(start_date[0])  # gets index of the current date
            # this will append the necessary codes into each OT array
            position = value_list.index(row[4])
            current_schedule[date_index, ot_list.index(row[5]), start_index:end_index] = 2000000 + (
            key_list[position]) * 1000

    # check current_dates and add 'temp_days' and 'current_dates'
    if (datetime.strptime(current_dates[-1], '%d/%m/%Y').date()).strftime('%A') == 'Friday':
        x1 = (datetime.strptime(current_dates[-1], '%d/%m/%Y').date())
        x2 = temp_days[-1]
        temp_array = np.zeros([1, 23, 44], dtype=np.int32)
        dict_elem_count = len(current_dates)
        mas_week_count = int(day_of_week_index[dict_elem_count - 1].split('y')[1])
        if mas_week_count != 5:
            mas_week_count = mas_week_count + 1
        else:
            mas_week_count = 1
        for i in range(3, 8):
            new_date = x1 + timedelta(days=i)
            new_datez = new_date.strftime("%d/%m/%Y")
            new_day = new_date.strftime('%A')
            current_dates.append(new_datez)
            new_dayfromnow = x2 + i
            temp_days.append(new_dayfromnow)
            current_schedule = np.concatenate([current_schedule, temp_array], axis=0)
            day_of_week_index[dict_elem_count] = str(new_day) + str(mas_week_count)
            dict_elem_count = dict_elem_count + 1

    current_schedule_list.append(current_schedule)
    current_schedule_list.append(temp_days)
    current_schedule_list.append(current_dates)
    current_schedule_list.append(day_of_week_index)

    return current_schedule_list


import numpy as np
from Generate_initial_solution import initial_solution_first_fit
from Generate_initial_solution import initial_solution_random_fit
from MO_fitting_func_calc import MO_calculate
from MO_fitting_func_calc import calculate_delta_S
from MO_fitting_func_calc import calc_crowding_distance
import find_legal_neighbors as fd
import swapper as sp
import pareto as pt
import genetic_algo_functions as ga
from read_MAS import read_mas
from read_csv import read_csv
from read_toschedule import read_toschedule
from plot_pareto_front import plot_front
import time
import random
import copy
from view_timetable1 import show_timetable
from threading import Thread
from output_table import create_table


def run_ga(current_schedule_list, to_schedule_read, mas_read):

    OT_indexes = {0: 'L1',
                  1: 'L2',
                  2: 'L3',
                  3: 'L4',
                  4: 'L5',
                  5: 'L6',
                  6: 'L7',
                  7: 'L8',
                  8: 'M1',
                  9: 'M2',
                  10: 'M3',
                  11: 'M4',
                  12: 'M5',
                  13: 'OT 24',
                  14: 'OT 25',
                  15: 'OT 22',
                  16: 'R1',
                  17: 'R4',
                  18: 'R5',
                  19: 'R6',
                  20: 'R7',
                  21: 'R8',
                  22: 'MRI',
                  }
    current_schedule = current_schedule_list
    # add one more schedule_date
    schedule_date = current_schedule[2]
    # code on top ^
    schedule_days = current_schedule[1]
    day_index = current_schedule[3]
    current_schedule = current_schedule[0]
    to_schedule = to_schedule_read
    # current_schedule = read_csv()
    # current_schedule = current_schedule[0]
    empty_schedule = copy.deepcopy(current_schedule)
    # to_schedule = read_toschedule()
    MAS_full = mas_read
    # MAS_full = read_mas()

    MAS_allowed = MAS_full['OTO']

    # first fit or random fit
    print("GENETIC ALGORITHM")
    n_pop = 30
    list_soln = []
    list_scores = []
    soln_list = []
    iter_ = 50
    t0 = time.time()
    # Generate population of solutions first
    for i in range(n_pop):
        current_schedule_temp = copy.deepcopy(current_schedule)
        soln = initial_solution_random_fit(current_schedule_temp, to_schedule, MAS_allowed, day_index, OT_indexes)
        score = MO_calculate(soln).tolist()
        list_soln.append(soln)
        list_scores.append(score)

    old_pop = list_soln
    # do non domintated sort
    pareto_ranks, pareto_sorted_scores, pareto_sorted_solns = ga.pareto_ranking(list_scores, old_pop, n_pop)
    for i in range(iter_):
        child_pop = []
        for _ in range(n_pop):
            # parent selection
            p1, p2 = ga.select_two_parent(pareto_ranks, list_scores, old_pop)
            # crossover + mutation
            child, clashes = ga.GA_crossover(p1, p2, to_schedule, empty_schedule)
            child = ga.GA_mutate(child, clashes, MAS_allowed, OT_indexes, day_index)
            child_pop.append(child)
        total_pop = old_pop + child_pop
        # calculate the new population scores, which consists of all new and old populations
        list_scores = []
        for soln in total_pop:
            list_scores.append(MO_calculate(soln).tolist())

        pareto_ranks, pareto_sorted_scores, pareto_sorted_solns = ga.pareto_ranking(list_scores, total_pop, n_pop)
        # take top n_pop solutions
        old_pop = pareto_sorted_solns[0:n_pop]
        list_scores = pareto_sorted_scores[0:n_pop]
        print(pareto_sorted_scores[0:n_pop])

    a = copy.deepcopy(list_scores)
    non_dom_scores = np.array(pt.eps_sort(list_scores))
    print("Pareto Front Scores:")
    print(non_dom_scores)

    # find minimum overtime score
    min_over_timescore = non_dom_scores[np.argmin(non_dom_scores[:, 0]), :]
    print("Minimum Overtime Score")
    print(min_over_timescore)
    min_index = np.where((pareto_sorted_scores == min_over_timescore).all(axis=1))[0][0]
    soln = pareto_sorted_solns[min_index]
    t1 = time.time()
    print("Time Taken for GA: " + str(t1 - t0) + ' seconds')

    # sol_list = [soln, schedule_days]
    sol_list = [soln, schedule_days, schedule_date]
    thread1 = Thread(target=show_timetable, args=[sol_list])
    thread1.start()
    # thread2 = Thread(target=create_table, args=(soln, schedule_date))
    # thread2.start()

    plot_front(np.array(a), non_dom_scores)


import numpy as np
from Generate_initial_solution import initial_solution_first_fit
from Generate_initial_solution import initial_solution_random_fit
from MO_fitting_func_calc import MO_calculate
from MO_fitting_func_calc import calc_crowding_distance
import find_legal_neighbors as fd
import swapper as sp
import pareto as pt
from read_MAS import read_mas
from read_csv import read_csv
from read_toschedule import read_toschedule
from plot_pareto_front import plot_front
import time
import random
from view_timetable import show_timetable
from threading import Thread
from output_table import create_table


def schedule_HCO(random_fit=True):
	OT_indexes = {0: 'L1',
	              1: 'L2',
	              2: 'L3',
	              3: 'L4',
	              4: 'L5',
	              5: 'L6',
	              6: 'L7',
	              7: 'L8',
	              8: 'M1',
	              9: 'M2',
	              10: 'M3',
	              11: 'M4',
	              12: 'M5',
	              13: 'OT 24',
	              14: 'OT 25',
	              15: 'OT 22',
	              16: 'R1',
	              17: 'R4',
	              18: 'R5',
	              19: 'R6',
	              20: 'R7',
	              21: 'R8',
	              22: 'MRI',
	              }
	current_schedule = read_csv()
	schedule_days = current_schedule[1]
	day_index = current_schedule[3]
	current_schedule = current_schedule[0]
	to_schedule = read_toschedule()
	MAS_full = read_mas()
	MAS_allowed = MAS_full['OTO']
	t0 = time.time()
	# first fit or random fit
	if random_fit:
		soln = initial_solution_random_fit(current_schedule, to_schedule, MAS_allowed, day_index, OT_indexes)
	else:
		soln = initial_solution_first_fit(current_schedule, to_schedule, MAS_allowed, day_index, OT_indexes)
	# HILL CLIMBING ALGORITHM
	print("Hill Climbing Algorithm")
	pastsolnscores = np.array([[0, 0, 0]])
	iter = 0
	while True:

		sp.pushback2(soln)
		curr_score = MO_calculate(soln)

		# generate all legal swaps
		legal_swaps = fd.all_legal_swaps(soln, to_schedule, MAS_allowed, day_index, OT_indexes)
		scores = np.array([[9999, 9999, 9999]])

		for swaps in legal_swaps:  # calculate score vector for each swap
			sc = MO_calculate(sp.swapping(soln, swaps))
			scores = np.vstack((scores, sc))

		scores = np.vstack((scores, curr_score))
		scores = np.delete(scores, 0, axis=0)
		p_front_scores = np.array(pt.eps_sort(scores))

		# find the best swap
		if not np.equal(curr_score, p_front_scores).all(
				axis=1).any():  # current solution is not in best possible pareto front
			crowding_dist = calc_crowding_distance(p_front_scores)
			min_crowding_dist_index = np.argmin(crowding_dist)
			find = p_front_scores[min_crowding_dist_index]
			min_index = np.where((scores == find).all(axis=1))[0][0]
			soln = sp.swapping(soln, legal_swaps[min_index])
			best_score = MO_calculate(soln)
		else:
			break  # current solution is already pareto optimal

		# check if the solution is looping
		if len(pastsolnscores) < 3:
			pastsolnscores = np.vstack((pastsolnscores, best_score))
		elif len(pastsolnscores) == 3:
			pastsolnscores = np.delete(pastsolnscores, 0, axis=0)
			pastsolnscores = np.vstack((pastsolnscores, best_score))
			if len(np.unique(pastsolnscores, axis=0)) < 3:
				break
		print("the best score is " + str(best_score))
		iter += 1
	print("Pareto Front Scores:")
	print(p_front_scores)
	t1 = time.time()

	# find minimum overtime score
	min_over_timescore = p_front_scores[np.argmin(p_front_scores[:, 0]), :]
	print("Minimum Overtime Score")
	print(min_over_timescore)
	min_index = np.where((scores == min_over_timescore).all(axis=1))[0][0]
	if min_index != len(scores)-1:
		soln = sp.swapping(soln, legal_swaps[min_index])
	print("Time Taken for HCO: " + str(t1-t0) + ' seconds')
	sol_list = [soln, schedule_days]
	thread = Thread(target=show_timetable, args=[sol_list])
	thread.start()
	plot_front(scores, p_front_scores)


import numpy as np
from Generate_initial_solution import initial_solution_first_fit
from Generate_initial_solution import initial_solution_random_fit
from MO_fitting_func_calc import MO_calculate
from MO_fitting_func_calc import calc_crowding_distance
import find_legal_neighbors as fd
import swapper as sp
import pareto as pt
from read_MAS import read_mas
from read_csv import read_csv
from read_toschedule import read_toschedule
from plot_pareto_front import plot_front
import time
import random
# from view_timetable1 import show_timetable
from view_timetable1 import show_timetable
from threading import Thread
from output_table import create_table

def run_hco(current_schedule_list, to_schedule_read, mas_read):

    OT_indexes = {0: 'L1',
                  1: 'L2',
                  2: 'L3',
                  3: 'L4',
                  4: 'L5',
                  5: 'L6',
                  6: 'L7',
                  7: 'L8',
                  8: 'M1',
                  9: 'M2',
                  10: 'M3',
                  11: 'M4',
                  12: 'M5',
                  13: 'OT 24',
                  14: 'OT 25',
                  15: 'OT 22',
                  16: 'R1',
                  17: 'R4',
                  18: 'R5',
                  19: 'R6',
                  20: 'R7',
                  21: 'R8',
                  22: 'MRI',
                  }

    current_schedule = current_schedule_list
    # add one more schedule_date
    schedule_date = current_schedule[2]
    # code on top ^
    schedule_days = current_schedule[1]
    day_index = current_schedule[3]
    current_schedule = current_schedule[0]
    to_schedule = to_schedule_read

    MAS_full = mas_read
    MAS_allowed = MAS_full['OTO']

    t0 = time.time()

    # first fit or random fit
    soln = initial_solution_first_fit(current_schedule, to_schedule, MAS_allowed, day_index, OT_indexes)

    # HILL CLIMBING ALGORITHM
    print("Hill Climbing Algorithm")
    pastsolnscores = np.array([[0, 0, 0]])
    iter = 0
    while True:

        sp.pushback2(soln)
        curr_score = MO_calculate(soln)

        # generate all legal swaps
        legal_swaps = fd.all_legal_swaps(soln, to_schedule, MAS_allowed, day_index, OT_indexes)
        scores = np.array([[9999, 9999, 9999]])

        for swaps in legal_swaps:  # calculate score vector for each swap
            sc = MO_calculate(sp.swapping(soln, swaps))
            scores = np.vstack((scores, sc))

        scores = np.vstack((scores, curr_score))
        scores = np.delete(scores, 0, axis=0)
        p_front_scores = np.array(pt.eps_sort(scores))

        # find the best swap
        if not np.equal(curr_score, p_front_scores).all(
                axis=1).any():  # current solution is not in best possible pareto front
            crowding_dist = calc_crowding_distance(p_front_scores)
            min_crowding_dist_index = np.argmin(crowding_dist)
            find = p_front_scores[min_crowding_dist_index]
            min_index = np.where((scores == find).all(axis=1))[0][0]
            soln = sp.swapping(soln, legal_swaps[min_index])
            best_score = MO_calculate(soln)
        else:
            break  # current solution is already pareto optimal

        # check if the solution is looping
        if len(pastsolnscores) < 3:
            pastsolnscores = np.vstack((pastsolnscores, best_score))
        elif len(pastsolnscores) == 3:
            pastsolnscores = np.delete(pastsolnscores, 0, axis=0)
            pastsolnscores = np.vstack((pastsolnscores, best_score))
            if len(np.unique(pastsolnscores, axis=0)) < 3:
                break
        print("the best score is " + str(best_score))
        iter += 1
    print("Pareto Front Scores:")
    print(p_front_scores)
    t1 = time.time()

    # find minimum overtime score
    min_over_timescore = p_front_scores[np.argmin(p_front_scores[:, 0]), :]
    print("Minimum Overtime Score")
    print(min_over_timescore)
    min_index = np.where((scores == min_over_timescore).all(axis=1))[0][0]
    if min_index != len(scores) - 1:
        soln = sp.swapping(soln, legal_swaps[min_index])
    print("Time Taken for HCO: " + str(t1 - t0) + ' seconds')

    # sol_list = [soln, schedule_days]
    # thread1 = Thread(target=show_timetable, args=[sol_list])
    # thread1.start()
    # thread2 = Thread(target=create_table, args=(soln, schedule_date))
    # thread2.start()
    sol_list = [soln, schedule_days, schedule_date]
    thread1 = Thread(target=show_timetable, args=[sol_list])
    thread1.start()

    plot_front(scores, p_front_scores)


import numpy as np
import math


def MO_calculate(soln):
	over_time_score = 0
	idle_time_score = 0
	waiting_time_score = 0
	alpha = 2.0  # multiplier for actual surgeries
	beta = 1.0  # multiplier for predicted surgeries

	# overtime
	last2hrs = soln[:, :, -8:]
	overtime_surg = last2hrs[last2hrs != 0]
	overtime_surg_type = [float(str(x)[1:2]) for x in overtime_surg]
	overtime_surg_type = np.array(overtime_surg_type)
	overtime_surg_type[overtime_surg_type == 1] *= alpha
	overtime_surg_type[overtime_surg_type == 2] *= beta
	over_time_score = sum(overtime_surg_type * 15)

	# idletime
	first9hrs = soln[:, :, :-8]
	zero_count = np.count_nonzero(first9hrs == 0, axis=2)
	days_away = np.arange(start=1, stop=len(zero_count) + 1)
	days_away = np.exp(days_away*-0.1)
	zero_count_t = zero_count.transpose()
	res = np.multiply(zero_count_t, days_away)
	res = res.flatten()
	idle_time_score = sum(res)

	# waiting time:
	inserted_surg = np.where((soln//100000>=20), 0, soln)
	inserted_surg = inserted_surg.reshape(len(inserted_surg), -1)
	inserted_surg[:, 1:] *= (np.diff(inserted_surg, axis=1) != 0)
	days_away = np.arange(start=1, stop=len(zero_count) + 1)
	inserted_surg = np.where((inserted_surg//100000==12),beta, inserted_surg)
	inserted_surg = np.where((inserted_surg//100000==11), alpha, inserted_surg)
	inserted_surg_t = inserted_surg.transpose()
	res_w = np.multiply(inserted_surg_t, days_away)
	res_w = res_w.flatten()
	waiting_time_score = sum(res_w)

	return np.array([over_time_score, idle_time_score, waiting_time_score])


def MO_calculate_diff(soln, curr_score, swap):

	alpha = 2.0  # multiplier for actual surgeries
	beta = 1.0  # multiplier for predicted surgeries

	score_vec1 = []
	score_vec2 = []
	swap1name = swap[0][0]
	swap1day = swap[0][1]
	swap1OT = swap[0][2]
	swap1start = swap[0][3]
	swap1end = swap[0][4]
	swap2name = swap[1][0]
	swap2day = swap[1][1]
	swap2OT = swap[1][2]
	swap2start = swap[1][3]
	swap2end = swap[1][4]
	# calculate scores for first slot
	overtime1 = soln[swap1day][swap1OT][swap1start]
	#idletime1 =
	if int(str(swap1name)[:2]) == 11:
		waitingtime1 = swap1day * alpha
	elif int(str(swap1name)[:2]) == 12:
		waitingtime1 = swap1day * beta
	#else:
		print("Warning: not 11 nor 12")
	# calculate scores for secon slot


def calc_crowding_distance(F):
	infinity = 1e+14

	n_points = F.shape[0]
	n_obj = F.shape[1]

	if n_points <= 2:
		return np.full(n_points, infinity)
	else:

		# sort each column and get index
		I = np.argsort(F, axis=0, kind='mergesort')

		# now really sort the whole array
		F = F[I, np.arange(n_obj)]

		# get the distance to the last element in sorted list and replace zeros with actual values
		dist = np.concatenate([F, np.full((1, n_obj), np.inf)]) \
		       - np.concatenate([np.full((1, n_obj), -np.inf), F])

		index_dist_is_zero = np.where(dist == 0)

		dist_to_last = np.copy(dist)
		for i, j in zip(*index_dist_is_zero):
			dist_to_last[i, j] = dist_to_last[i - 1, j]

		dist_to_next = np.copy(dist)
		for i, j in reversed(list(zip(*index_dist_is_zero))):
			dist_to_next[i, j] = dist_to_next[i + 1, j]

		# normalize all the distances
		norm = np.max(F, axis=0) - np.min(F, axis=0)
		norm[norm == 0] = np.nan
		dist_to_last, dist_to_next = dist_to_last[:-1] / norm, dist_to_next[1:] / norm

		# if we divided by zero because all values in one columns are equal replace by none
		dist_to_last[np.isnan(dist_to_last)] = 0.0
		dist_to_next[np.isnan(dist_to_next)] = 0.0

		# sum up the distance to next and last and norm by objectives - also reorder from sorted list
		J = np.argsort(I, axis=0)
		crowding = np.sum(dist_to_last[J, np.arange(n_obj)] + dist_to_next[J, np.arange(n_obj)], axis=1) / n_obj

	# replace infinity with a large number
	crowding[np.isinf(crowding)] = infinity

	return crowding


def calculate_delta_S(curr, neighbor, T): # scalarizing function
	lambda_o = 1
	lambda_i = 5
	lambda_w = 5
	lambda_ = [lambda_o, lambda_i, lambda_w]
	diff_vector = MO_calculate(neighbor) - MO_calculate(curr)
	pi = np.where(diff_vector <= 0, 1, np.exp(-diff_vector/T))
	v = np.power(pi, lambda_)
	return np.prod(v)


import numpy as np
from Generate_initial_solution import initial_solution_first_fit
from Generate_initial_solution import initial_solution_random_fit
from MO_fitting_func_calc import MO_calculate
from MO_fitting_func_calc import calculate_delta_S
from MO_fitting_func_calc import calc_crowding_distance
import find_legal_neighbors as fd
import swapper as sp
import pareto as pt
from read_MAS import read_mas
from read_csv import read_csv
from read_toschedule import read_toschedule
import plot_pareto_front as pf
import time
import random
from view_timetable import show_timetable
from threading import Thread


def schedule_TS(k=5, tenure=5, iter_max=50):

	OT_indexes = {0: 'L1',
	              1: 'L2',
	              2: 'L3',
	              3: 'L4',
	              4: 'L5',
	              5: 'L6',
	              6: 'L7',
	              7: 'L8',
	              8: 'M1',
	              9: 'M2',
	              10: 'M3',
	              11: 'M4',
	              12: 'M5',
	              13: 'OT 24',
	              14: 'OT 25',
	              15: 'OT 22',
	              16: 'R1',
	              17: 'R4',
	              18: 'R5',
	              19: 'R6',
	              20: 'R7',
	              21: 'R8',
	              22: 'MRI',
	              }
	current_schedule = read_csv()
	schedule_days = current_schedule[1]
	day_index = current_schedule[3]
	current_schedule = current_schedule[0]
	to_schedule = read_toschedule()
	MAS_full = read_mas()
	MAS_allowed = MAS_full['OTO']
	t0 = time.time()
	k = k  # number of parallel searches
	tabu_tenure = tenure
	soln_list = []
	for _ in range(k):
		soln_list.append(
			initial_solution_random_fit(current_schedule, to_schedule, MAS_allowed, day_index, OT_indexes))

	pareto_front = np.array([[9999,9999,9999]])
	max_iter = iter_max
	tabu_list = [[None] for i in range(k)]

	# PARALLEL TABU SEARCH ALGORITHM
	print("TABU SEARCH Algorithm")
	# Trying to esimate pareto front from multiple initial solutions

	for iter in range(max_iter):
		new_search_front_scores = []
		new_search_front = []
		scores_t = np.array([[9999, 9999, 9999]])
		for index, soln in enumerate(soln_list):
			sp.pushback2(soln)
			curr_score = MO_calculate(soln)
			# generate all legal swaps
			legal_swaps = fd.all_legal_swaps_subset(soln, to_schedule, MAS_allowed, day_index, OT_indexes) # use a random subsetted neighbour to make program run faster
			scores = np.array([[9999, 9999, 9999]])
			for swaps in legal_swaps:  # bottleneck
				sc = MO_calculate(sp.swapping(soln, swaps))
				scores = np.vstack((scores, sc))
			scores = np.delete(scores, 0, axis=0)
			scores_t = np.concatenate((scores_t, scores))
			p_front_scores_x = pf.pareto_ranking(
				scores)  # not really pareto front, with some that are not on pareto front, but also quite good
			pareto_front = np.concatenate((pareto_front, p_front_scores_x),axis=0) # will have to squeeze later

			c = random.choice(p_front_scores_x)
			c_index = np.where((scores==c).all(axis=1))[0][0]
			c_soln = sp.swapping(soln, legal_swaps[c_index])
			while c in tabu_list[index]:
				print('tabu!')
				c = random.choice(p_front_scores_x)
				c_index = np.where((scores == c).all(axis=1))[0][0]
				c_soln = sp.swapping(soln, legal_swaps[c_index])
			new_search_front_scores.append(c)  # the next iteration's soln list
			new_search_front.append(c_soln)
			# update tabu list
			tabu_list[index].pop(0)
			tabu_list[index].extend(c*tabu_tenure)
		scores_t = np.delete(scores_t,0, axis=0)
		print(new_search_front_scores)
		soln_list = new_search_front


	pareto_front = np.delete(pareto_front,0,axis=0)
	pareto_front_final = np.array(pt.eps_sort(pareto_front))
	print("Pareto Front Scores:")
	print(pareto_front_final)

	# find minimum overtime score
	min_over_timescore = pareto_front_final[np.argmin(pareto_front_final[:, 0]), :]
	print("Minimum Overtime Score")
	print(min_over_timescore)
	t1 = time.time()
	print("Time Taken for TS: " + str(t1-t0) + ' seconds')
	sol_list = [soln, schedule_days]
	thread = Thread(target=show_timetable, args=[sol_list])
	thread.start()
	pf.plot_front(scores, pareto_front_final)


from mpl_toolkits.mplot3d import axes3d
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.tri as mtri
import pareto as pt
import copy

def plot_front2(dp, pp, iter):

	fig = plt.figure()
	ax = fig.add_subplot(111, projection='3d')

	ax.scatter(dp[:,0],dp[:,1],dp[:,2])
	ax.scatter(pp[:,0],pp[:,1],pp[:,2],color='red')

	triang = mtri.Triangulation(pp[:, 0], pp[:, 1])
	ax.plot_trisurf(triang, pp[:, 2], color='red')
	ax.set_xlim(0, 2500)
	ax.set_ylim(650, 720)
	ax.set_zlim(70, 160)
	#plt.show()
	plt.savefig('plt' + str(iter) + '.png')

	return

def plot_front(dp, pp):

	fig = plt.figure()
	ax = fig.add_subplot(111, projection='3d')

	ax.scatter(dp[:,0],dp[:,1],dp[:,2])
	ax.scatter(pp[:,0],pp[:,1],pp[:,2],color='red')

	triang = mtri.Triangulation(pp[:, 0], pp[:, 1])
	ax.plot_trisurf(triang, pp[:, 2], color='red')
	plt.show()
	return

def pareto_ranking(scores1):
	scores = copy.deepcopy(scores1).tolist()
	rankswanted = 5
	n = 10
	pareto_chosen = []
	for _ in range(rankswanted):
		pareto_efficient_pts = pt.eps_sort(scores)
		for sc in pareto_efficient_pts:
			pareto_chosen.append(sc)
		scores = [x for x in scores if x not in pareto_efficient_pts]
	return pareto_chosen[0:n]